@@ -127,7 +127,12 @@ static struct oonf_timer_instance _transmission_timer = {
   .class = &_transmission_timer_info
 };
 
-static uint32_t _l2_origin;
+static struct oonf_layer2_origin _l2_origin = {
+  .name = "ethernet listener",
+  .priority = OONF_LAYER2_ORIGIN_UNRELIABLE,
+  .proactive = true,
+};
+
 static int _ioctl_sock;
 
 static int
@@ -139,14 +144,14 @@ _init(void) {
   }
 
   oonf_timer_add(&_transmission_timer_info);
-  _l2_origin = oonf_layer2_register_origin();
+  oonf_layer2_add_origin(&_l2_origin);
 
   return 0;
 }
 
 static void
 _cleanup(void) {
-  oonf_layer2_cleanup_origin(_l2_origin);
+  oonf_layer2_remove_origin(&_l2_origin);
 
   oonf_timer_stop(&_transmission_timer);
   oonf_timer_remove(&_transmission_timer_info);
@@ -198,6 +203,14 @@ _cb_transmission_event(struct oonf_timer_instance *ptr __attribute((unused))) {
       continue;
     }
 
+    /* get ethernet linkspeed */
+    ethspeed = ethtool_cmd_speed(&cmd);
+    if (ethspeed == 0 || ethspeed == (uint16_t)-1 || ethspeed == (uint32_t)-1) {
+      /* speed is not known */
+      continue;
+    }
+    ethspeed *= 1000 * 1000;
+
     /* layer-2 object for this interface */
     l2net = oonf_layer2_net_add(os_if->name);
     if (l2net == NULL) {
@@ -207,19 +220,15 @@ _cb_transmission_event(struct oonf_timer_instance *ptr __attribute((unused))) {
       l2net->if_type = OONF_LAYER2_TYPE_ETHERNET;
     }
 
-    /* get ethernet linkspeed */
-    ethspeed = ethtool_cmd_speed(&cmd);
-    ethspeed *= 1000 * 1000;
-
     /* set corresponding database entries */
     OONF_DEBUG(LOG_ETH, "Set default link speed of interface %s to %s",
         os_if->name,
         isonumber_from_s64(&ibuf, ethspeed, "bit/s", 0, false, false));
 
     oonf_layer2_set_value(&l2net->neighdata[OONF_LAYER2_NEIGH_RX_BITRATE],
-        _l2_origin, ethspeed);
+        &_l2_origin, ethspeed);
     oonf_layer2_set_value(&l2net->neighdata[OONF_LAYER2_NEIGH_TX_BITRATE],
-        _l2_origin, ethspeed);
+        &_l2_origin, ethspeed);
   }
 }
 
