@@ -492,6 +492,7 @@ _cb_interface_event(struct oonf_rfc5444_interface_listener *ifl,
   struct os_interface_listener *if_listener;
   struct nhdp_link *nhdp_link, *nhdp_link_it;
   struct os_interface_ip *os_ip;
+  const union netaddr_socket *sock;
   bool has_active_addr;
   bool ipv4, ipv6;
 #ifdef OONF_LOG_DEBUG_INFO
@@ -511,8 +512,10 @@ _cb_interface_event(struct oonf_rfc5444_interface_listener *ifl,
 
   if_listener = oonf_rfc5444_get_core_if_listener(ifl->interface);
   if (if_listener != NULL && if_listener->data && if_listener->data->flags.up) {
-    ipv4 = oonf_rfc5444_is_target_active(interf->rfc5444_if.interface->multicast4);
-    ipv6 = oonf_rfc5444_is_target_active(interf->rfc5444_if.interface->multicast6);
+    ipv4 = if_listener->data->flags.loopback
+        || oonf_rfc5444_is_target_active(interf->rfc5444_if.interface->multicast4);
+    ipv6 = if_listener->data->flags.loopback
+        || oonf_rfc5444_is_target_active(interf->rfc5444_if.interface->multicast6);
 
     /* get all socket addresses that are matching the filter */
     avl_for_each_element(&if_listener->data->addresses, os_ip, _node) {
@@ -550,4 +553,20 @@ _cb_interface_event(struct oonf_rfc5444_interface_listener *ifl,
       nhdp_db_link_set_unsymmetric(nhdp_link);
     }
   }
+
+  /* get local IPv4 socket address */
+  netaddr_invalidate(&interf->local_ipv4);
+  sock = oonf_rfc5444_interface_get_local_socket(
+      interf->rfc5444_if.interface, AF_INET);
+  if (sock) {
+    netaddr_from_socket(&interf->local_ipv4, sock);
+  }
+
+  /* get local IPv6 socket address */
+  netaddr_invalidate(&interf->local_ipv6);
+  sock = oonf_rfc5444_interface_get_local_socket(
+      interf->rfc5444_if.interface, AF_INET6);
+  if (sock) {
+    netaddr_from_socket(&interf->local_ipv6, sock);
+  }
 }
