@@ -82,7 +82,8 @@
 
 EXPORT uint64_t olsrv2_get_tc_interval(void);
 EXPORT uint64_t olsrv2_get_tc_validity(void);
-EXPORT const struct netaddr_acl *olsrv2_get_routable(void);
+EXPORT bool olsrv2_is_nhdp_routable(struct netaddr *addr);
+EXPORT bool olsrv2_is_routable(struct netaddr *addr);
 EXPORT bool olsrv2_mpr_shall_process(
     struct rfc5444_reader_tlvblock_context *, uint64_t vtime);
 EXPORT bool olsrv2_mpr_shall_forwarding(
