@@ -39,6 +39,10 @@
  *
  */
 
+/**
+ * @file
+ */
+
 #include "common/avl.h"
 #include "common/avl_comp.h"
 #include "common/common_types.h"
@@ -46,8 +50,8 @@
 #include "common/netaddr_acl.h"
 #include "core/oonf_logging.h"
 #include "subsystems/oonf_class.h"
-#include "subsystems/oonf_interface.h"
 #include "subsystems/oonf_timer.h"
+#include "subsystems/os_interface.h"
 
 #include "nhdp/nhdp.h"
 
@@ -58,7 +62,8 @@
 static struct olsrv2_originator_set_entry *_remember_removed_originator(
     struct netaddr *originator, uint64_t vtime);
 static void _set_originator(int af_type, struct netaddr *setting, const struct netaddr *new_originator);
-static void _cb_originator_entry_vtime(void *);
+static void _cb_originator_entry_vtime(struct oonf_timer_instance *);
+static void _remove_originator_entry(struct olsrv2_originator_set_entry *entry);
 
 /* originator set class and timer */
 static struct oonf_class _originator_entry_class = {
@@ -99,7 +104,7 @@ olsrv2_originator_cleanup(void) {
 
   /* remove all originator entries */
   avl_for_each_element_safe(&_originator_set_tree, entry, _node, e_it) {
-    _cb_originator_entry_vtime(entry);
+    _remove_originator_entry(entry);
   }
 
   /* remove timer and class */
@@ -154,6 +159,10 @@ olsrv2_originator_set(const struct netaddr *originator) {
   }
 }
 
+/**
+ * Get tree of old originator IPs
+ * @return originator tree
+ */
 struct avl_tree *
 olsrv2_originator_get_tree(void) {
   return &_originator_set_tree;
@@ -184,7 +193,6 @@ _remember_removed_originator(struct netaddr *originator, uint64_t vtime) {
 
     /* initialize timer */
     entry->_vtime.class = &_originator_entry_timer;
-    entry->_vtime.cb_context = entry;
   }
 
   /* reset validity time */
@@ -217,7 +225,7 @@ _set_originator(int af_type, struct netaddr *setting, const struct netaddr *new_
   /* remove new_originator originator from set */
   entry = olsrv2_originator_get_entry(new_originator);
   if (entry) {
-    _cb_originator_entry_vtime(entry);
+    _remove_originator_entry(entry);
   }
 
   /* update NHDP originator */
@@ -231,12 +239,18 @@ _set_originator(int af_type, struct netaddr *setting, const struct netaddr *new_
 
 /**
  * Callback fired when originator set entry must be removed
- * @param ptr pointer to originator set
+ * @param ptr timer instance that fired
  */
 static void
-_cb_originator_entry_vtime(void *ptr) {
-  struct olsrv2_originator_set_entry *entry = ptr;
+_cb_originator_entry_vtime(struct oonf_timer_instance *ptr) {
+  struct olsrv2_originator_set_entry *entry;
 
+  entry = container_of(ptr, struct olsrv2_originator_set_entry, _vtime);
+  _remove_originator_entry(entry);
+}
+
+static void
+_remove_originator_entry(struct olsrv2_originator_set_entry *entry) {
   oonf_timer_stop(&entry->_vtime);
   avl_remove(&_originator_set_tree, &entry->_node);
 
