@@ -39,6 +39,10 @@
  *
  */
 
+/**
+ * @file
+ */
+
 #include "common/common_types.h"
 #include "common/avl.h"
 #include "common/avl_comp.h"
@@ -58,6 +62,12 @@
 /* Definitions */
 #define LOG_TELNET _oonf_telnet_subsystem.logging
 
+struct _telnet_config {
+  struct oonf_stream_managed_config osmc;
+  int32_t allowed_sessions;
+  uint64_t timeout;
+};
+
 /* static function prototypes */
 static int _init(void);
 static void _cleanup(void);
@@ -77,22 +87,25 @@ static enum oonf_telnet_result _telnet_handle_command(
 static struct oonf_telnet_command *_check_telnet_command_acl(
     struct oonf_telnet_data *data, struct oonf_telnet_command *cmd);
 
-static void _cb_telnet_repeat_timer(void *data);
+static void _cb_telnet_repeat_timer(struct oonf_timer_instance *data);
 static enum oonf_telnet_result _cb_telnet_quit(struct oonf_telnet_data *data);
 static enum oonf_telnet_result _cb_telnet_help(struct oonf_telnet_data *data);
 static enum oonf_telnet_result _cb_telnet_echo(struct oonf_telnet_data *data);
 static enum oonf_telnet_result _cb_telnet_repeat(struct oonf_telnet_data *data);
 static enum oonf_telnet_result _cb_telnet_timeout(struct oonf_telnet_data *data);
-static enum oonf_telnet_result _cb_telnet_version(struct oonf_telnet_data *data);
 
 /* configuration of telnet server */
 static struct cfg_schema_entry _telnet_entries[] = {
-  CFG_MAP_ACL_V46(oonf_stream_managed_config,
-      acl, "acl", ACL_DEFAULT_ACCEPT, "Access control list for telnet interface"),
-  CFG_MAP_ACL(oonf_stream_managed_config,
-      bindto, "bindto", "127.0.0.1\0" "::1\0" ACL_DEFAULT_REJECT, "Allowed addressed to bind telnet socket to"),
-  CFG_MAP_INT32_MINMAX(oonf_stream_managed_config,
-      port, "port", "2009", "Network port for telnet interface", 0, false, 1, 65535),
+  CFG_MAP_ACL_V46(_telnet_config, osmc.acl,
+      "acl", ACL_DEFAULT_ACCEPT, "Access control list for telnet interface"),
+  CFG_MAP_ACL(_telnet_config, osmc.bindto,
+      "bindto", "127.0.0.1\0" "::1\0" ACL_DEFAULT_REJECT, "Allowed addressed to bind telnet socket to"),
+  CFG_MAP_INT32_MINMAX(_telnet_config, osmc.port,
+      "port", "2009", "Network port for telnet interface", 0, false, 1, 65535),
+  CFG_MAP_INT32_MINMAX(_telnet_config, allowed_sessions,
+      "allowed_sessions", "3", "Maximum number of allowed simultaneous sessions",0, false, 3, 1024),
+  CFG_MAP_CLOCK(_telnet_config, timeout,
+      "timeout", "120000", "Time until a telnet session is closed when idle"),
 };
 
 static struct cfg_schema_section _telnet_section = {
@@ -110,12 +123,11 @@ static struct oonf_telnet_command _builtin[] = {
   TELNET_CMD("exit", _cb_telnet_quit, "Ends telnet session"),
   TELNET_CMD("help", _cb_telnet_help,
       "help: Display the online help text and a list of commands"),
-  TELNET_CMD("echo", _cb_telnet_echo,"echo <string>: Prints a string"),
+  TELNET_CMD("echo", _cb_telnet_echo, "echo <string>: Prints a string"),
   TELNET_CMD("repeat", _cb_telnet_repeat,
       "repeat <seconds> <command>: Repeats a telnet command every X seconds"),
   TELNET_CMD("timeout", _cb_telnet_timeout,
       "timeout <seconds> :Sets telnet session timeout"),
-  TELNET_CMD("version", _cb_telnet_version, "Displays version of the program"),
 };
 
 /* subsystem definition */
@@ -149,7 +161,6 @@ static struct oonf_timer_class _telnet_repeat_timerinfo = {
 static struct oonf_stream_managed _telnet_managed = {
   .config = {
     .session_timeout = 120000, /* 120 seconds */
-    .maximum_input_buffer = 4096,
     .allowed_sessions = 3,
     .memcookie = &_telnet_memcookie,
     .init = _cb_telnet_init,
@@ -250,7 +261,7 @@ oonf_telnet_execute(const char *cmd, const char *para,
   data.remote = remote;
 
   result = _telnet_handle_command(&data);
-  oonf_telnet_stop(&data, false);
+  _call_stop_handler(&data);
   return abuf_has_failed(data.out) ? TELNET_RESULT_INTERNAL_ERROR : result;
 }
 
@@ -279,32 +290,6 @@ _avl_comp_strcmdword(const void *ptr1, const void *ptr2) {
   return diff;
 }
 
-/**
- * Handler for configuration changes
- */
-static void
-_cb_config_changed(void) {
-  struct oonf_stream_managed_config config;
-
-  /* generate binary config */
-  memset(&config, 0, sizeof(config));
-  if (cfg_schema_tobin(&config, _telnet_section.post,
-      _telnet_entries, ARRAYSIZE(_telnet_entries))) {
-    /* error in conversion */
-    OONF_WARN(LOG_TELNET, "Cannot map telnet config to binary data");
-    goto apply_config_failed;
-  }
-
-  if (oonf_stream_apply_managed(&_telnet_managed, &config)) {
-    /* error while updating sockets */
-    goto apply_config_failed;
-  }
-
-  /* fall through */
-apply_config_failed:
-  oonf_stream_free_managed_config(&config);
-}
-
 /**
  * Initialization of incoming telnet session
  * @param session pointer to TCP session
@@ -419,17 +404,18 @@ _cb_telnet_receive_data(struct oonf_stream_session *session) {
 
     /* search for end of line */
     eol = memchr(abuf_getptr(&session->in), '\n', abuf_getlen(&session->in));
-
-    if (eol == NULL) {
+    if (eol) {
+      /* terminate line with a 0 */
+      if (eol != abuf_getptr(&session->in) && eol[-1] == '\r') {
+        eol[-1] = 0;
+      }
+      *eol++ = 0;
+    }
+    else if (session->state == STREAM_SESSION_ACTIVE) {
+      /* more data might be coming */
       break;
     }
 
-    /* terminate line with a 0 */
-    if (eol != abuf_getptr(&session->in) && eol[-1] == '\r') {
-      eol[-1] = 0;
-    }
-    *eol++ = 0;
-
     /* handle line */
     OONF_DEBUG(LOG_TELNET, "Interactive console: %s\n", abuf_getptr(&session->in));
     cmd = abuf_getptr(&session->in);
@@ -461,8 +447,15 @@ _cb_telnet_receive_data(struct oonf_stream_session *session) {
         OONF_DEBUG(LOG_TELNET, "Processing telnet command: '%s' '%s'",
             cmd, para);
 
-        telnet_session->data.command = cmd;
-        telnet_session->data.parameter = para;
+        if (para != NULL && strcmp(para, "help") == 0) {
+          /* switch command and parameter to allow "<cmd> help" variant */
+          telnet_session->data.command = para;
+          telnet_session->data.parameter = cmd;
+        }
+        else {
+          telnet_session->data.command = cmd;
+          telnet_session->data.parameter = para;
+        }
 
         cmd_result = _telnet_handle_command(&telnet_session->data);
         if (abuf_has_failed(telnet_session->data.out)) {
@@ -498,9 +491,11 @@ _cb_telnet_receive_data(struct oonf_stream_session *session) {
     }
 
     /* remove line from input buffer */
-    abuf_pull(&session->in, eol - abuf_getptr(&session->in));
+    if (eol) {
+      abuf_pull(&session->in, eol - abuf_getptr(&session->in));
+    }
 
-    if (abuf_getptr(&session->in)[0] == '/') {
+    if (chainCommands) {
       /* end of multiple command line */
       return STREAM_SESSION_SEND_AND_QUIT;
     }
@@ -656,8 +651,7 @@ _cb_telnet_timeout(struct oonf_telnet_data *data) {
  */
 static void
 _cb_telnet_repeat_stophandler(struct oonf_telnet_data *data) {
-  oonf_timer_stop((struct oonf_timer_instance *)data->stop_data[0]);
-  free(data->stop_data[0]);
+  oonf_timer_stop(&data->stop_timer);
   free(data->stop_data[1]);
 
   data->stop_handler = NULL;
@@ -668,13 +662,15 @@ _cb_telnet_repeat_stophandler(struct oonf_telnet_data *data) {
 
 /**
  * Timer event handler for repeating telnet commands
- * @param ptr pointer to custom data
+ * @param ptr timer instance that fired
  */
 static void
-_cb_telnet_repeat_timer(void *ptr) {
-  struct oonf_telnet_data *telnet_data = ptr;
+_cb_telnet_repeat_timer(struct oonf_timer_instance *ptr) {
+  struct oonf_telnet_data *telnet_data;
   struct oonf_telnet_session *session;
 
+  telnet_data = container_of(ptr, struct oonf_telnet_data, stop_timer);
+
   /* set command/parameter with repeat settings */
   telnet_data->command = telnet_data->stop_data[1];
   telnet_data->parameter = telnet_data->stop_data[2];
@@ -695,7 +691,6 @@ _cb_telnet_repeat_timer(void *ptr) {
  */
 static enum oonf_telnet_result
 _cb_telnet_repeat(struct oonf_telnet_data *data) {
-  struct oonf_timer_instance *timer;
   int interval = 0;
   char *ptr = NULL;
 
@@ -717,17 +712,10 @@ _cb_telnet_repeat(struct oonf_telnet_data *data) {
     return TELNET_RESULT_ACTIVE;
   }
 
-  timer = calloc(1, sizeof(*timer));
-  if (timer == NULL) {
-    return TELNET_RESULT_INTERNAL_ERROR;
-  }
-
-  timer->cb_context = data;
-  timer->class = &_telnet_repeat_timerinfo;
-  oonf_timer_start(timer, MSEC_PER_SEC * interval);
+  data->stop_timer.class= &_telnet_repeat_timerinfo;
+  oonf_timer_start(&data->stop_timer, (uint64_t)MSEC_PER_SEC * interval);
 
   data->stop_handler = _cb_telnet_repeat_stophandler;
-  data->stop_data[0] = timer;
   data->stop_data[1] = strdup(ptr);
   data->stop_data[2] = NULL;
 
@@ -751,12 +739,31 @@ _cb_telnet_repeat(struct oonf_telnet_data *data) {
 }
 
 /**
- * Telnet command 'version'
- * @param data pointer to telnet data
- * @return telnet command result
+ * Handler for configuration changes
  */
-static enum oonf_telnet_result
-_cb_telnet_version(struct oonf_telnet_data *data) {
-  oonf_log_printversion(data->out);
-  return TELNET_RESULT_ACTIVE;
+static void
+_cb_config_changed(void) {
+  struct _telnet_config config;
+
+  /* generate binary config */
+  memset(&config, 0, sizeof(config));
+  if (cfg_schema_tobin(&config, _telnet_section.post,
+      _telnet_entries, ARRAYSIZE(_telnet_entries))) {
+    /* error in conversion */
+    OONF_WARN(LOG_TELNET, "Cannot map telnet config to binary data");
+    goto apply_config_failed;
+  }
+
+  /* set session parameters */
+  _telnet_managed.config.allowed_sessions = config.allowed_sessions;
+  _telnet_managed.config.session_timeout = config.timeout;
+
+  if (oonf_stream_apply_managed(&_telnet_managed, &config.osmc)) {
+    /* error while updating sockets */
+    goto apply_config_failed;
+  }
+
+  /* fall through */
+apply_config_failed:
+  oonf_stream_free_managed_config(&config.osmc);
 }
